#ifndef __ALCHEMIST_PRIVATE_H__
#define __ALCHEMIST_PRIVATE_H__

#define ADM_VERSION_STRING "0"

/* flag bit values */
#define ADM_DATA_FLAG_PROTECTED	(1 << 0)
#define ADM_DATA_FLAG_ATOMIC	(1 << 1)
#define ADM_DATA_FLAG_PREPEND	(1 << 2)
#define ADM_DATA_FLAG_ANONYMOUS	(1 << 3)

/*
 * Note that AdmData, and AdmIdentity have their context element
 * as the first element. This allows this macro to work on AdmData,
 * AdmIdentity, AdmList, AdmInt, AdmFloat, AdmBool, AdmString,
 * AdmBase64, and AdmCopy type pointers, so don't change it!
 */
#define ADM_NODE_ISALIVE(x) (((AdmData *) x)->context != NULL)

/* AdmData and sub types */
struct AdmData {
	AdmContext *context; /* must be first, see ADM_NODE_ISALIVE */

	AdmRefStr *name;	/* must never be NULL */
	AdmRefStr *source;	/* must never be NULL */

	/* data tree pointers */
	AdmData *parent;
	AdmData *child;		/* only valid for AdmLists */
	AdmData *next;		/* the list of siblings is circular */
	AdmData *prev;
	
	gint16 flags;

	guint refcount;
	AdmDataType type;
};
struct AdmList { AdmData meta; guint num_children; };
struct AdmBool { AdmData meta; gboolean value; };
struct AdmInt { AdmData meta; gint value; };
struct AdmFloat { AdmData meta; gdouble value; };
struct AdmString { AdmData meta; AdmRefStr *value; };

/* AdmIdentity */
struct AdmIdentity {
	AdmContext *context; /* must be first, see ADM_NODE_ISALIVE */
	
	AdmRefStr *name;	/* must never be NULL */

	AdmIdentity *parentA;
	AdmIdentity *parentB;

	guint serial;
	guint refcount;
};

/* AdmContext */
struct AdmContext {
	AdmIdentity *id_tree_root;
	AdmData *data_tree_root;
	GSList *mask_set;
	guint mask_set_size;
	guint refcount;
};

/* Private Functions */

/* path functions */

/* this is the fastest I could bum ISALPHA and friends, isn't it neat? */
#define ADM_ISALPHA(c) ({ char j, k; j = c | 0x20; k = (j >= 'a') & (j <= 'z'); })
#define ADM_ISDIGIT(c) ({ char j, k; j = c; k = (j >= '0') & (j <= '9'); })
#define ADM_ISNAMECHAR(c) (ADM_ISALPHA(c) | ADM_ISDIGIT(c) | (c == '_') | (c == '-') | (c == '.'))
gboolean	_adm_valid_name_by_length	(const gchar *name,
						 guint len);
#define _adm_valid_name(X) _adm_valid_name_by_length(X,strlen(X))

gboolean	_adm_valid_path	(const gchar *path);

int	_adm_valid_base64 (const char *string, unsigned long len);

/* data tree functions */
AdmData*	_adm_malloc_data	(AdmDataType type);
void	_adm_free_data		(AdmData *data);

AdmData*	_adm_create_data	(AdmContext     *context,
					 AdmData        *parent,
					 AdmDataType     type,
					 AdmRefStr *name,
					 AdmRefStr *source);

AdmData*	_adm_copy_data_tree	(AdmContext *context,
				   	 AdmData    *parent,
				   	 AdmData    *data,
					 gboolean    strip);

AdmData*	_adm_path_lookup	(AdmList *list,
					 const char *path);

void     _adm_list_merge         (AdmList  *listX,
                                         AdmList  *listY,
					 AdmCopy **head,
					 AdmCopy **tail);

/* id tree functions */
AdmIdentity*	_adm_malloc_id	(void);
void		_adm_free_id	(AdmIdentity *id);

AdmIdentity*	_adm_create_identity	(AdmContext *context,
						 AdmRefStr *name,
						 guint serial);

void		_adm_unlink_id_tree	(AdmIdentity *identity);

AdmIdentity*	_adm_copy_id_tree	(AdmContext *context,
						 AdmIdentity *id);

#endif/*__ALCHEMIST_PRIVATE_H__*/
